# coding: utf-8

"""
    Antimatter Public API

    Interact with the Antimatter Cloud API

    The version of the OpenAPI document: 2.0.1
    Contact: support@antimatter.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictBytes, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from antimatter_api.models.read_context_config_rule import ReadContextConfigRule
from antimatter_api.models.read_context_parameter import ReadContextParameter
from antimatter_api.models.read_context_required_hook import ReadContextRequiredHook
from typing import Optional, Set
from typing_extensions import Self

class ReadContextDetails(BaseModel):
    """
    Details about a read context 
    """ # noqa: E501
    name: Annotated[str, Field(strict=True)] = Field(description="A human readable name for a read context")
    summary: Annotated[str, Field(strict=True, max_length=140)] = Field(description="A short, single sentence description of this read context")
    description: Annotated[str, Field(strict=True, max_length=4096)] = Field(description="A longer form description of this read context")
    disable_read_logging: StrictBool = Field(description="If true, skip creation of audit log events on read", alias="disableReadLogging")
    key_cache_ttl: Annotated[int, Field(strict=True, ge=0)] = Field(description="Number of seconds for which cached encryption keys will be considered valid by the client ", alias="keyCacheTTL")
    required_hooks: Optional[List[ReadContextRequiredHook]] = Field(default=None, alias="requiredHooks")
    read_parameters: List[ReadContextParameter] = Field(alias="readParameters")
    rules: List[ReadContextConfigRule]
    imported: StrictBool = Field(description="true if this read context is imported")
    source_domain_id: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="A globally unique identifier for a domain", alias="sourceDomainID")
    source_domain_name: Optional[StrictStr] = Field(default=None, alias="sourceDomainName")
    policy_assembly: Optional[Union[StrictBytes, StrictStr]] = Field(default=None, description="a WASM bundle version of the policy associated with this read context ", alias="policyAssembly")
    __properties: ClassVar[List[str]] = ["name", "summary", "description", "disableReadLogging", "keyCacheTTL", "requiredHooks", "readParameters", "rules", "imported", "sourceDomainID", "sourceDomainName", "policyAssembly"]

    @field_validator('name')
    def name_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^((dm-[1-9A-HJ-NP-Za-km-z]{11}|[a-z][a-z0-9_]{2,31})::)?[a-z][a-z0-9_]{2,31}$", value):
            raise ValueError(r"must validate the regular expression /^((dm-[1-9A-HJ-NP-Za-km-z]{11}|[a-z][a-z0-9_]{2,31})::)?[a-z][a-z0-9_]{2,31}$/")
        return value

    @field_validator('source_domain_id')
    def source_domain_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^dm-[1-9A-HJ-NP-Za-km-z]{11}$", value):
            raise ValueError(r"must validate the regular expression /^dm-[1-9A-HJ-NP-Za-km-z]{11}$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ReadContextDetails from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in required_hooks (list)
        _items = []
        if self.required_hooks:
            for _item in self.required_hooks:
                if _item:
                    _items.append(_item.to_dict())
            _dict['requiredHooks'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in read_parameters (list)
        _items = []
        if self.read_parameters:
            for _item in self.read_parameters:
                if _item:
                    _items.append(_item.to_dict())
            _dict['readParameters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in rules (list)
        _items = []
        if self.rules:
            for _item in self.rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['rules'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ReadContextDetails from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "summary": obj.get("summary"),
            "description": obj.get("description"),
            "disableReadLogging": obj.get("disableReadLogging") if obj.get("disableReadLogging") is not None else False,
            "keyCacheTTL": obj.get("keyCacheTTL") if obj.get("keyCacheTTL") is not None else 0,
            "requiredHooks": [ReadContextRequiredHook.from_dict(_item) for _item in obj["requiredHooks"]] if obj.get("requiredHooks") is not None else None,
            "readParameters": [ReadContextParameter.from_dict(_item) for _item in obj["readParameters"]] if obj.get("readParameters") is not None else None,
            "rules": [ReadContextConfigRule.from_dict(_item) for _item in obj["rules"]] if obj.get("rules") is not None else None,
            "imported": obj.get("imported"),
            "sourceDomainID": obj.get("sourceDomainID"),
            "sourceDomainName": obj.get("sourceDomainName"),
            "policyAssembly": obj.get("policyAssembly")
        })
        return _obj


